'-----------------------------------------------------------------------
' <copyright file="SBAForm.vb" company="Microsoft">
'     Copyright (c) Microsoft Corporation.  All rights reserved.
' </copyright>
' <disclaimer>
'  THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESSED
'  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
'  MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK OF
'  USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE REMAINS WITH THE USER.  
' </disclaimer>
'-----------------------------------------------------------------------

Public Class SBAForm
    Friend sbaObjects As ISbaObjects
    Friend smallBusinessInstance As ISmallBusinessInstance

    ' A new DataTable to store ICustomerAccount property values
    Friend itemProperties As DataTable
    ' the Type of the object whose properties are listed in the itemPropertiesGridView
    Friend currentItem As IItemAccount

    ' The name of a .resx file containing localizable UI resources
    '  NOTE - SBAForm.resx is generated by the designer.  SBAResources.resx
    '  is a safer place to store non-designer-generated resources.
    Const resourceFILE As String = "SBASolution.SBAResources"
    ' Resource manager for localizable UI resources
    Friend isvResources As ResourceManager

    ' Initialize the WinForm and set sbaObjects
    ' 
    ' This constructor is used to create an SBAForm within an existing AppDomain
    ' (ISbaObjects can be passed in directly)
    Public Sub New(ByVal sbaObjects As ISbaObjects)
        ' set sbaObjects
        Me.sbaObjects = sbaObjects

        ' Initialize form
        InitializeSBAForm()
    End Sub

    ' Initialize the WinForm and set sbaObjects
    ' 
    ' This constructor is used to create an SBAForm within a new AppDomain (since
    ' parameters must be serializable to pass across an AppDomain boundary)
    Public Sub New(ByVal databaseName As String, ByVal serverName As String)
        ' Define the full names of the Loader assembly and class. These values should
        ' match the values defined in SBAStandardCode.cs.
        Const loaderFULLNAME As String = "Loader, Version=2.0.5201.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"
        Const loaderNAMESPACE As String = "Microsoft.BusinessSolutions.SmallBusinessAccounting.Loader.Loader"

        ' Reload the Loader assembly
        Dim loaderAssembly As Assembly = Assembly.Load(loaderFULLNAME)

        ' Instantiate ILoader object
        Dim loader As ILoader = DirectCast(loaderAssembly.CreateInstance(loaderNAMESPACE), ILoader)

        ' Instantiate sbaObjects object
        Me.sbaObjects = loader.GetSbaObjects(databaseName, serverName)

        ' Initialize Form
        InitializeSBAForm()
    End Sub

    ' Initialize common elements
    Friend Sub InitializeSBAForm()

        Dim itemGroupView As IItemGroupView
        Dim itemRow As DataRowView
        Dim itemGroup As IItemGroup
        Dim customerPrimaryKeys(0) As DataColumn

        ' Initialize WinForm
        InitializeComponent()

        ' Load resource file
        isvResources = New ResourceManager(resourceFILE, Assembly.GetExecutingAssembly())

        ' set ISmallBusinessInstance, IFormsFactory
        Me.smallBusinessInstance = DirectCast(Me.sbaObjects.SmallBusinessInstance, ISmallBusinessInstance)

        ' Populate the item group list box
        itemGroupView = smallBusinessInstance.ItemGroups
        For Each itemRow In itemGroupView.DataView
            itemGroup = itemGroupView.GetByDataRow(itemRow.Row)
            itemGroupListBox.Items.Add(itemGroup)
        Next
        ' Add a checkbox entry for items without item groups            
        itemGroupListBox.Items.Add(isvResources.GetString("resItemGroupNone"))

        ' Initialize data objects and bind to customerPropertiesGridView
        '  Create new DataTable
        '  Create DataColumns for the DataTable
        '  Designate one column as a PrimaryKey so that DataTable rows can be retrieved by name
        itemProperties = New DataTable()
        itemProperties.Locale = CultureInfo.CurrentCulture
        customerPrimaryKeys(0) = itemProperties.Columns.Add("Name")
        itemProperties.Columns.Add("Value")
        itemProperties.PrimaryKey = customerPrimaryKeys
        itemPropertiesGridView.DataSource = itemProperties
        itemPropertiesGridView.Sort(itemPropertiesGridView.Columns("Name"), System.ComponentModel.ListSortDirection.Ascending)

    End Sub

    ' Start the message loop
    '
    ' Application.Run(Form) must be run from within the local AppDomain
    ' in order for the message loop to start cleanly.
    Public Sub Run()
        Application.Run(Me)
    End Sub

    Private Sub SBAForm_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        ' Log off the Small Business Accounting database
        Me.smallBusinessInstance.LogOff()
    End Sub

    ' Update the List<IItemAccount> to reflect the new check status, whatever it is
    Private Sub itemGroupListBox_ItemCheck(ByVal sender As Object, ByVal e As ItemCheckEventArgs) Handles itemGroupListBox.ItemCheck

        Dim itemView As IItemView
        Dim displayedItems As ListBox.ObjectCollection
        Dim addedGroup As IItemGroup
        Dim checkItem As Boolean
        Dim itemAccount As IItemAccount
        Dim itemGroup As IItemGroup
        Dim rowView As DataRowView
        Dim currentCursor As Cursor

        ' Set the current cursor to an hourglass
        currentCursor = Me.Cursor
        Me.Cursor = Cursors.WaitCursor

        ' The SBA item account view
        itemView = smallBusinessInstance.ItemAccounts
        ' The collection of objects in the item list box
        displayedItems = itemListBox.Items
        ' The item group (or Nothing, if the item is the string that indicates "none")
        '  that has just been checked or unchecked
        addedGroup = TryCast(itemGroupListBox.Items(e.Index), IItemGroup)

        ' Add or remove the items in the currently selected item group to or from the item
        '  list box

        ' NOTE:  This is a performant way to iterate through any view that implements
        '  IBaseMasterView
        For Each rowView In itemView.DataView

            ' Get the IItemAccount corresponding to the data row
            itemAccount = DirectCast(itemView.GetByDataRow(rowView.Row), IItemAccount)

            ' Get the item group
            '  Kit items require special handling because they do not implement
            '  an accessor for IItemAccount.ItemGroup
            If TypeOf itemAccount Is IKitItemAccount Then
                itemGroup = Nothing
            Else
                itemGroup = itemAccount.ItemGroup
            End If

            ' If the current item has the same item group as the one that was just
            '  checked or unchecked, add or remove it from the item list.

            checkItem = False

            If itemGroup Is Nothing Then
                If addedGroup Is Nothing Then
                    checkItem = True
                End If
            Else
                If addedGroup IsNot Nothing Then
                    If itemGroup.Guid = addedGroup.Guid Then
                        checkItem = True
                    End If
                End If
            End If

                If checkItem Then

                    ' e.NewValue is always equal to CheckState.Checked when ShowAll is true.
                    If ((e.NewValue = CheckState.Checked) And (displayedItems.Contains(itemAccount) = False)) Then

                        ' Add the item to the displayed item list
                        displayedItems.Add(itemAccount)
                    End If


                    If ((e.NewValue = CheckState.Unchecked) And (itemListBox.Items.Contains(itemAccount))) Then

                        Try
                            ' Remove the item from the displayed item list
                            displayedItems.Remove(itemAccount)
                            ' ListBox.Remove() throws a NothingReferenceException if the selected item is removed
                        Catch ex As NullReferenceException
                            ' Clear the displayed item properties
                            itemProperties.Clear()
                        End Try
                    End If
                End If
        Next

        ' Make sure the changes are displayed in the list box.
        itemListBox.Refresh()

        ' Reset the cursor
        Me.Cursor = currentCursor

    End Sub

    Private Sub itemListBox_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) Handles itemListBox.SelectedIndexChanged

        Dim listBox As ListBox
        Dim itemAccount As IItemAccount

        listBox = sender

        ' Get the item account.
        '  NOTE:  since the Windows Forms control is based on a collection of objects
        '  (in this case instances of IItemAccount), it is only necessary to 
        '  cast sender.SelectedItem to the appropriate type.
        itemAccount = DirectCast(listBox.SelectedItem, IItemAccount)

        ' If the object type changes, the property control needs to be cleared.

        If currentItem IsNot Nothing Then
            If currentItem.GetEntityType() IsNot itemAccount.GetEntityType() Then
                itemProperties.Clear()
            End If
        End If

        currentItem = itemAccount

        ' Update itemPropertyGridView using the selected item account            
        UpdateItemPropertyView(itemAccount)
    End Sub

    Private Sub UpdateItemPropertyView(ByVal itemAccount As IItemAccount)

        ' An array of strings to provide values for new data rows
        Dim paramList(1) As String
        ' An object that stores the value of each property of the ICustomerAccount instance
        Dim propertyValue As Object
        Dim propertyNum As Decimal
        Dim propertyRow As DataRow

        Dim itemProperty As PropertyInfo

        If itemAccount Is Nothing Then
            Return
        End If

        ' Update the DataTable with the value of each ICustomerAccount property.
        '  If the property is not already in the DataTable, add it.
        ' NOTE:  This code assumes that the Type of object that is used to populate the 
        '  DataTable is always the same.  If that is not the case, call customerProperties.Clear()
        '  before processing each new object.
        For Each itemProperty In itemAccount.GetEntityType().GetProperties()

            ' Store the name of the property
            paramList(0) = itemProperty.Name

            ' Get the value of the property (converted to a String)
            Try
                propertyValue = itemProperty.GetValue(itemAccount, Nothing)

            Catch e As Exception

                ' Some IDocumentLine properties are meaningless for IKitLine instances,
                '  since IKitItems represent collections rather than single items.  Those 
                '  properties throw ISmallBusinessException when they are accessed.
                If (TypeOf e.InnerException Is ISmallBusinessException) Then
                    propertyValue = e.InnerException.ToString()
                Else
                    Throw
                End If
            End Try

            ' Store the value of the property.  If the value is Nothing, store String.Empty.
            If propertyValue IsNot Nothing Then

                ' Display the value of the property.  If the property is a decimal type, format
                '  it as currency for display.
                If TypeOf propertyValue Is Decimal Then

                    propertyNum = propertyValue
                    paramList(1) = propertyNum.ToString("C", CultureInfo.CurrentCulture) ' Treat Decimal values as local currency

                Else
                    paramList(1) = propertyValue.ToString()
                End If
            Else
                paramList(1) = String.Empty
            End If

            ' If the property is not in the DataTable, add it
            ' If it is in the DataTable, update its value
            propertyRow = itemProperties.Rows.Find(paramList(0))
            If (propertyRow Is Nothing) Then
                itemProperties.Rows.Add(paramList)
            Else
                propertyRow(1) = paramList(1)
            End If
        Next
    End Sub
End Class