'-----------------------------------------------------------------------
' <copyright file="SBAInitialization.vb" company="Microsoft">
'     Copyright (c) Microsoft Corporation.  All rights reserved.
' </copyright>
' <disclaimer>
'  THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESSED
'  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
'  MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK OF
'  USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE REMAINS WITH THE USER.  
' </disclaimer>
'-----------------------------------------------------------------------

Partial Public Class SBAAppLoader

   ' NOTE: The SBAAppLoader class should not include references to interfaces 
   ' defined in any SBA assembly except ILoader.dll.

   ' Global variable declarations
   '

   ' The minimum version of Small Business Accounting required to run the WinForm
   '
   ' This value should be consistent with the assemblies referenced by the WinForm class.
   ' For example, if the WinForm references SBAIAPIV2.dll, the requiredMajorVersion should
   ' be at least 2.
   Private Const requiredMajorVersion As Integer = 2

   ' The Small Business Accounting version of the connected database
   Private Shared runtimeMajorVersion As Integer

   ' The culture required to run the WinForm ("" if there are no restrictions)
   '
   ' This value should be consistent with the assemblies referenced by the WinForm class.
   ' For example, if the WInForm references SBAIAPIENGB.dll, the requiredCulture should be
   ' "en-gb".
   Private Const requiredCulture As String = ""

   ' The culture of the connected database
   Private Shared runtimeCulture As String

   ' Whether or not the WinForm will run ina separate AppDomain (always true when
   ' connected to a V1 database)
   Private Shared createInAppDomain As Boolean = False

   ' The name of the default Small Business company file (String.Empty or "" if there is
   ' no default company file.
   Private Const defaultSBCFile As String = "sampleproductcompany.sbc"

   <STAThread()> Public Shared Sub Main()
      ' Provide default values for runtime variables

      ' The Type fo of the default SBA form. The form (or its base class) must implement
      ' the following methods:
      '
      '    .ctor(ISbaObjects)       A constructor that accepts an ISbaObjects instance
      '    .ctor(string, string)    A constructor that accepts a database name and a
      '                             server name and uses them to load the Loader and to
      '                             attach to a database.
      '    Run()                    A method that runs the form.
      '
      ' The default implementation of these members is provided in SBAForm.vb.
      Dim sbaFormType As Type = GetType(SBAForm)

      ' The base directory used when the form is run in a new AppDomain. Its typical value
      ' (provided by GetSBAInstallationLocation) is the parent directory of the Small
      ' Business Accounting directory.
      Dim sbaBaseDirectory As String = String.Empty

      ' The relative search path used when the form is run in a new AppDomain. It should
      ' include the Small Business Accounting directory and the ISV application directory.
      Dim assemblySearchPath As String = String.Empty

      ' Set the default application settings for a WinForm
      Application.EnableVisualStyles()
      Application.SetCompatibleTextRenderingDefault(False)

      ' Attach to a database, load SBA objects and set global fields (including
      ' createInAppDomain=true if runtimeMajorVersion is 1.)  
      ' Exit if the Loader is not able to attach to a database.

      If LoadSbaObjects(defaultSBCFile) Is Nothing Then Return

      ' Provide base directory and search path if the new form will be launched
      ' in a new AppDomain
      If createInAppDomain Then
         sbaBaseDirectory = GetSBAInstallationLocation()
         assemblySearchPath = GetAssemblySearchPath()
      End If

      ' Launch the SBAForm          
      RunISVApplication(sbaFormType, sbaBaseDirectory, assemblySearchPath)
   End Sub

   ' GetAssemblySearchPath
   '
   ' Defines the relative search path used by a new AppDomain to probe for
   ' non-GACced assemblies. It returns a list of partial paths separated by
   ' semicolons.
   ' 
   ' The partial paths are subfolders of the base directory of the AppDomain.
   ' The base directory is provided by GetSBAInstallationLocation().
   '
   ' By default, the base directory is "C:\Program Files\Microsoft Small Business"
   ' and SBA V1 is installed in the "Small Business Accounting" subdirectory.
   ' 
   ' If all relevant assemblies (including SBA) are in the GAC, this method can
   ' return Nothing
   Private Shared Function GetAssemblySearchPath() As String
      Const sbaSUBFOLDER As String = "Small Business Accounting;"

      ' Use this value if the ISV assemblies are in a subdirectory of the SBA base directory.
      ' e.g., "\Program FIles\Microsoft Small Business\ISVApplication"
      Dim isvSubfolder As String = Path.GetFileName(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location))

      ' Use this value if the ISV assemblies are in the GAC
      ' Dim isvSubfolder As String = String.Empty

      Return sbaSUBFOLDER & isvSubfolder
   End Function
End Class
